#ifndef STM32_USART_H
#define STM32_USART_H

/*{ stm32_usart.h **********************************************************
 
                      The ToolChain
                      
   Universal Synchronous Asynchronous Receiver and Transmitters (USARTs)
   for ARM CortexM3 microcontrollers.
   Currently tested to comply to STM32F10x types.
   
   written by Gregor Rebel 2012

 
}*/
//{ Defines/ TypeDefs ****************************************************

//} Defines
//{ Includes *************************************************************

#include "ttc_usart_types.h"
#include "ttc_task.h"

 // Basic set of helper functions
#include "stm32_gpio.h"
#include "stm32_registers.h"
#include <stdlib.h>

#define USE_STDPERIPH_DRIVER
#include "stm32f10x.h"
#include "core_cm3.h"        // StdPeripheralsLibrary/CMSIS/CM3/CoreSupport/
#include "stm32f10x_conf.h"
#include "stm32f10x_usart.h"
#include "misc.h"

/*{ optional
// ADC
#include "stm32f10x_adc.h"
#include "stm32f10x_dma.h"

// Timers
#include "stm32f10x_tim.h"

#include "stm32f10x_it.h"
#include "system_stm32f10x.h"

#include "bits.h"
#include <string.h>
#include "FreeRTOS.h"
#include "task.h"
}*/

//} Includes
//{ Structures/ Enums ****************************************************

typedef struct {  // architecture specific configuration data of single USART 
  USART_TypeDef* Base;     // base address of USART registers
  stm32_Port_t PortTxD;    // port pin for TxD
  stm32_Port_t PortRxD;    // port pin for RxD
  stm32_Port_t PortRTS;    // port pin for RTS
  stm32_Port_t PortCTS;    // port pin for CTS
  stm32_Port_t PortCLK;    // port pin for CLK
} __attribute__((__packed__)) stm32_usart_architecture_t;

//} Structures/ Enums
//{ Function prototypes **************************************************

/* resets library. Automatically called.
 */
void stm32_usart_reset_all();

/* fills out given USART_ with default values for indexed USART
 * @param USART_Index     device index of USART to init (1..ttc_usart_get_max_index())
 * @param USART_Generic   pointer to struct ttc_usart_generic_t
 * @return  == 0:         *USART_Generic has been initialized successfully; != 0: error-code
 */
ttc_usart_errorcode_e stm32_usart_get_defaults(u8_t USART_Index, ttc_usart_generic_t* USART_Generic);

/* fills out given USART_Generic with maximum valid values for indexed USART
 * @param USART_Index     device index of USART to init (1..ttc_usart_get_max_index())
 * @param USART_Generic   pointer to struct ttc_usart_generic_t
 * @return  == 0:         *USART_Generic has been initialized successfully; != 0: error-code
 */
ttc_usart_errorcode_e stm32_usart_get_features(u8_t USART_Index, ttc_usart_generic_t* USART_Generic);

/* initializes single USART
 * @param USART_Index     device index of USART to init (1..ttc_usart_get_max_index())
 * @param USART_Generic   filled out struct ttc_usart_generic_t
 * @return  == 0:         USART has been initialized successfully; != 0: error-code
 */
ttc_usart_errorcode_e stm32_usart_init(u8_t USART_Index, ttc_usart_generic_t* USART_Generic, stm32_usart_architecture_t* USART_Arch);

/* Send out given amount of raw bytes from buffer.
 * Will not interpret bytes in buffer.
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until all bytes have been sent!
 * 
 * @param USART_Index    device index of USART to init (0..ttc_usart_get_max_index()-1)
 * @param Buffer         memory location from which to read data
 * @param Amount         amount of bytes to send from Buffer[]
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
ttc_usart_errorcode_e stm32_usart_send_raw(u8_t USART_Index, const char* Buffer, u16_t Amount);

/* Send out given bytes from buffer until first zero byte or MaxLength reached.
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until all bytes have been sent!
 * 
 * @param USART_Index    device index of USART to init (0..ttc_usart_get_max_index()-1)
 * @param Buffer         memory location from which to read data
 * @param MaxLength      no more than this amount of bytes will be send
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
ttc_usart_errorcode_e stm32_usart_send_string(u8_t USART_Index, const char* Buffer, u16_t MaxLength);

/* Send out given data word (8 or 9 bits)
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until all bytes have been sent!
 *
 * @param USART_Index    device index of USART to init (0..ttc_usart_get_max_index()-1)
 * @param Byte           8 or 9 bit value to send
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
ttc_usart_errorcode_e stm32_usart_send_word(u8_t USART_Index, const u16_t Word);

/* Reads single data word from input buffer (8 or 9 bits)
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until a valid word has been received or timeout occurs
 *
 * @param USART_Index    device index of USART to init (0..ttc_usart_get_max_index()-1)
 * @param Word           pointer to 16 bit buffer where to store Word
 * @param TimeOut        >0: max microseconds to wait; ==0: no timeout
 * @return               == 0: Word has been read successfully; != 0: error-code
 */
ttc_usart_errorcode_e stm32_usart_read_word(u8_t USART_Index, u16_t* Word, u32_t TimeOut);

/* Reads single data byte from input buffer.
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until a valid byte has been received or timeout occurs
 *
 * @param USART_Index    device index of USART to init (0..ttc_usart_get_max_index()-1)
 * @param Word           pointer to 16 bit buffer where to store Word
 * @param TimeOut        >0: max microseconds to wait; ==0: no timeout
 * @return               == 0: Word has been read successfully; != 0: error-code
 */
ttc_usart_errorcode_e stm32_usart_read_byte(u8_t USART_Index, char* Byte, u32_t TimeOut);

/* Sends out given byte.
 * Note: This low-level function should not be called from outside!
 */
void _stm32_usart_send_word(USART_t* MyUSART, const u16_t Word);

/* Wait until word has been received or timeout occured.
 * Note: This low-level function should not be called from outside!
 */
ttc_usart_errorcode_e _stm32_usart_wait_for_RXNE(USART_t* MyUSART, u32_t TimeOut);

//} Function prototypes

#endif //STM32_USART_H
