#ifndef ttc_task_H
#define ttc_task_H

/*{ ttc_task.h ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Architecture independent support for multitasking.
 * 
 * Currently implemented architectures: stm32 + FreeRTOS
 * 
}*/

//{ includes

#include "ttc_basic.h"
#include "ttc_gpio.h"

#ifdef EXTENSION_300_scheduler_free_rtos
  #include <string.h>
  #include "stddef.h"
  #include "FreeRTOS.h"
  #include "task.h"
#endif


//}includes

//{ Function prototypes **************************************************

/* produces software PWM usable to calibrate uSleep() 
 * Idea: Connect oscilloscope to pin PA1 (pin 15 on STM32F103) an check for correct period (1,1ms) and duty cycle.
 *        Adjust uSleep() accordingly, recompile, reflash and retest.
 */
void programCalibrateUSleep();

/* Set task to sleep for some amount of time
 * @param Delay    delay time (msecs)
*/
void ttc_task_msleep(u32_t Delay);
void mSleep(u32_t Delay);

/* set task to sleep for some amount of time
 * @param Delay    delay time (usecs)
 */
void ttc_task_usleep(u32_t Delay);
void uSleep(u32_t Delay);

/* spawns a function as separate task
 * @param TaskFunction  function to start as task (must never return!)
 * @param TaskName      used for debugging purposes only
 * @param StackSize     each task has its own size; choose wisely
 * @param Argument      passed as argument to TaskFunction()
 * @param Priority      tasks with higher priority get more cpu time
 * @param Handle        =! NULL: stores task handle in *Handle; A task handle can be used to destroy the task later.
 */
void ttc_task_create( void TaskFunction(void*), const signed char* TaskName, u16_t StackSize, void* Argument, u8_t Priority, void* Handle);


/* Switches scheduler to next process providing a short, undetermined sleep
 *
 * Some applications require a busy waiting for resources like:
 *   while (...);
 * During this loop, the cpu is completely busy with this loop. This blocks or at least slows down other tasks.
 * Even if this task can be interrupted by the task scheduler, it fills its time slice with checking its condition (...).
 *
 * A more multitask-friendly version is to yield cpu time to other processes while waiting:
 *   while (...) ttc_task_yield();
 * For every checked condition (...), every task gets one slice of cpu time. The processing overhead is minimal.
 *
 */
void ttc_task_yield();

/* Temporarily disables task scheduler*/
void ttc_task_begin_criticalsection();

/* Reenables task scheduler */
void ttc_task_end_criticalsection();

//}Function prototypes

#endif
